package spiral.all;

import spiral.test.*;
import java.io.*;
import java.awt.*;

/**
 * <p>Title: Spiral analyzis</p>
 * <p>Description: Tda reprezentujc spirlu - vektor obsahujc souadnice bod x a y, as a tlak</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Dolezel
 * @version 1.0
 */

public class Data {
  private DataVector rows = null;
  private Engine engine = null;
  private Configuration conf = null;
  private int maxX = -10000, minX = 10000, maxY = -10000, minY = 10000;

  /**
   * Nainicializuje nov vektor dat
   * @param en Engine, kter objekt d
   */
  public Data(Engine en, Configuration c) {
    engine = en;
    conf = c;
    rows = new DataVector();
  }

  /**
   * Vytvo nov vektor dat ze souboru file
   * @param file soubor, ze kterho se budou st data
   * @param en Engine, kter objekt d
   */
  public Data(String file, Engine en) {
    engine = en;
    String line = null;
    String[] str = null;
    int x, y, time, pres;
    BufferedReader in = null;
    DataRow r = null;
    rows = new DataVector(1000);

    try {
      in = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
      engine.gui.mShow("Soubor otevren");

      while ( (line = in.readLine()) != null) {
        if (line.trim().charAt(0) != '#') {
          str = line.split(" "); // jednotliv hodnoty oddleny mezerou
          if (str.length != 4) {
            engine.gui.mShow("Nestandartni vstupni soubor!");
            in.close();
            rows = null;
            return;
          }
          this.newRow(Integer.parseInt(str[0]), Integer.parseInt(str[1]),
                      Integer.parseInt(str[2]), Integer.parseInt(str[3])); // ulo dek
        }
        else {
          engine.gui.mShow("Komentar: " + str);
        }
      }

      in.close();
    }
    catch (IOException e) {
      engine.gui.mShow("IO Chyba");
    }
  }

  /**
   * Pid dal dku s daty do vektoru
   * @param x x-ov souadnice
   * @param y y-ov souadnice
   * @param time as
   * @param pres tlak
   */
  public void newRow(int x, int y, int time, int pres) {
    rows.add(new DataRow(x, y, time, pres));
    maxX = Math.max(maxX, x);
    minX = Math.min(minX, x);
    maxY = Math.max(maxY, y);
    minY = Math.min(minY, y);
  }

  /**
   * Vykresl spirlu po cel ploe g
   * @param g kam, se m spirla vykreslit
   */
  public void paint(Graphics g) {
    if ( (rows == null) || rows.size() < 2) {
      return;
    }
    int size = rows.size();
    int x1, x2, y1, y2;
    double pomx = (double) (engine.gui.getMaxX() - 10) / (maxX - minX); // koeficient X
    double pomy = (double) (engine.gui.getMaxY() - 10) / (maxY - minY); // koeficient Y
    double pom = (pomx > pomy) ? pomy : pomx; // vyber mensi, aby se to tam veslo
    x1 = (int) ( (rows.get(0).getX() - minX) * pom) + 4;
    y1 = (int) ( (rows.get(0).getY() - minY) * pom) + 4;
    g.setColor(Color.RED);
    g.fillRect(x1 - 1, y1 - 1, 3, 3);
    for (int i = 1; i < size; i++) { // pro vechny body
      x2 = (int) ( (rows.get(i).getX() - minX) * pom) + 4;
      y2 = (int) ( (rows.get(i).getY() - minY) * pom) + 4;
      g.setColor(Color.BLACK);
      g.drawLine(x1, y1, x2, y2); // ry mezi body
      g.setColor(Color.RED);
      g.fillRect(x2 - 1, y2 - 1, 3, 3); // zvrazni vlastn body
      x1 = x2;
      y1 = y2;
    }
  }

  /**
   * Vrac velikost vektoru dat
   * @return poet bod spirly
   */
  public int size() {
    if (rows != null) {
      return rows.size();
    }
    else {
      return 0;
    }
  }

  /**
   * Vrac vlastn reprezentaci vektoru, ve kterm jsou uloeny data
   * @return vektor dat
   */
  public DataVector getRows() {
    return rows;
  }

  /**
   * Ulo data do souboru
   * @param fileName jmno souboru
   */
  public void save(String fileName) {
    String str;
    try {
      BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new
          FileOutputStream(fileName)));
      int size = rows.size();
      for (int i = 1; i < size; i++) {
        str = rows.get(i).toString();
        out.write(str, 0, str.length());
        out.newLine();
      }
      out.close();
    }
    catch (IOException ex) {
      engine.gui.mShow("Nelze vytvorit soubor");
    }
  }

  public Configuration getConf() {
    return conf;
  }
}