package spiral.all;

import spiral.test.*;
import java.io.*;
import java.util.*;

/**
 * <p>Title: Spiral analyzis</p>
 * <p>Description: Tda reprezentujc vypotan hodnoty spirly/p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Dolezel
 * @version 1.0
 */

public class Results {
  private Engine eng = null;
  private ResultVector res = null;
  private Configuration conf;

  private double crfi = 0, c2rfi = 0, fi = 0, maxr = 0;
  private double I1 = 0, I2 = 0, I3 = 0, I4 = 0, I5 = 0, DOS = 0;

  /**
   * Spot DOS a ostatn parametry
   * @param e Engine, ovldajc tento objekt
   * @param data spirly, pro kterou m bt spotn vsledek
   */
  public Results(Engine e, Data data) {
    eng = e;
    conf = data.getConf();

    DataVector rows = data.getRows();
    int all = rows.size();
    res = new ResultVector(all);

    int cx = (int) rows.get(0).getX();
    int cy = (int) rows.get(0).getY();
    int dpi = eng.gui.getDPI();
    int step = 1000 / eng.gui.getHZ(); // casov krok mezi sousednmy body

    // promene vysvetleny dale nebo podle nazvu...
    double r1, r2, r3, dr, dfi, drfi, d2rfi, fi1, fi2, fipom, ldrfi;
    double x1, x2, y1, y2;

    // inicializace pro prvni smycku
    x1 = (rows.get(0).getX() - cx) / dpi;
    y1 = (rows.get(0).getY() - cy) / dpi;
    maxr = r1 = Math.sqrt(x1 * x1 + y1 * y1); // polomer bodu
    dfi = fi1 = Math.atan2(y1, x1); // uhel bodu
    d2rfi = ldrfi = drfi = 0; // minule hodnoty
    res.add(new ResultRow(x1, y1, rows.get(0).getTime(), rows.get(0).getPres(),
                          r1, fi, dfi, drfi, d2rfi));

    int time = rows.get(0).getTime();
    int omm = 0;

    for (int i = 1; i < all; i++) {
      if (rows.get(i).getTime() < time) {
        continue;
      }
      x2 = (rows.get(i).getX() - cx) / dpi;
      y2 = (rows.get(i).getY() - cy) / dpi;
      r2 = Math.sqrt(x2 * x2 + y2 * y2); // polomer bodu
      maxr = Math.max(maxr, r2);
      dr = r2 - r1; // vypocet delta r
      fipom = fi2 = Math.atan2(y2, x2); // uhel bodu
      if (Math.abs(fi2 - fi1) > Math.PI) { // prepocet pro uhly blizici se PI z ruznych stran
        if (fi1 < 0) {
          fi1 += 2 * Math.PI;
        }
        else {
          fipom += 2 * Math.PI;
        }
      }
      dfi = fipom - fi1; // vypocet delta fi
      if (Math.abs(dfi) > 0.000001) {
        drfi = dr / dfi; // 1. derivace r podle fi
        d2rfi = (drfi - ldrfi) / dfi; // 2. derivace r podle fi
        fi += dfi; // celkovy uhel
        res.add(new ResultRow(x2, y2, rows.get(i).getTime(),
                              rows.get(i).getPres(), r2, fi, dfi, drfi, d2rfi));
        if ( (Math.abs(drfi) > 1000) || (Math.abs(d2rfi) > 1000)) {
          omm++;
          continue; // odstraneni velkych vychylek
        }
        crfi += drfi; // suma 1. derivace
        c2rfi += d2rfi; // suma 2. derivace
      }

      // posun na dalsi radku
      r1 = r2;
      x1 = x2;
      y1 = y2;
      fi1 = fi2;
      ldrfi = drfi;
      time += step;
    }

    // vysledek inicializace
    int size = res.size();
    crfi /= size; // prumerna spirala
    c2rfi /= size; // odchylka od prumerne spiraly
    fi = Math.abs(fi);
    eng.gui.mShow("number of points: " + all + ", timedout: " + (all - size) + ", ommited: " + omm);
    eng.gui.mShow("crfi: " + crfi + ", c2rfi: " + c2rfi + ", fi: " + fi);

    // vlastn vpoet hodnot
    for (int i = 1; i < size; i++) {
      I1 += (res.get(i).drfi - crfi) * (res.get(i).drfi - crfi); // suma odchylky 1. derivace
      I2 += (res.get(i).d2rfi - c2rfi) * (res.get(i).d2rfi - c2rfi); // suma odchylky 2. derivace
      I4 +=
          Math.abs(sign(res.get(i).drfi - crfi) -
                   sign(res.get(i - 1).drfi - crfi)); // suma extremu
      I5 +=
          Math.abs(sign(res.get(i).d2rfi - c2rfi) -
                   sign(res.get(i - 1).d2rfi - c2rfi)); // suma inflexnich bodu
      // posun na dalsi radku
    }
    I1 = Math.log(I1 / fi) / Math.log(Math.E); // normovani na pocet otocek
    I2 = Math.log(I2 / fi) / Math.log(Math.E); // normovani na pocet otocek
    I3 = ( (fi / maxr) - (14 * Math.PI)) / (2 * Math.PI); // zhadn parametr ~ tsnost
    I4 = I4 / (size - 1) * 50; // procentuln poet maxim/minim
    I5 = I5 / (size - 1) * 50; // procentuln poet inflexnch bod
    DOS = (0.4615 * I1 + 0.0544 * I5 - 0.2331 * I1 * I1 - 0.0726 * I2 * I2 -
           0.001 * I5 * I5 + 0.2539 * I1 * I2 + 1.3668);
    eng.gui.mShow("I1: " + simpleRound(I1) + ",  I2: " + simpleRound(I2) +
                  ",  I3: " + simpleRound(I3) + ",  I4: " + simpleRound(I4) +
                  "%,  I5: " + simpleRound(I5) + "%");
    eng.gui.mShow("DOS: " + DOS);
  }

  private int sign(double x) {
    if (x == 0) {
      return 0;
    }
    else if (x > 0) {
      return 1;
    }
    else {
      return -1;
    }
  }

  private String simpleRound(double val, int prec) {
    int pow = (int) Math.pow(10, prec - 1);
    return String.valueOf(Math.rint(val * pow) / pow);
  }

  private String simpleRound(double val) {
    return simpleRound(val, 6);
  }

  /**
   * Ulo vypotan vsledky do souboru ve formtu <x> <y> <as> <tlak> <r> <\u03B8> <\u0394\u03B8> <\u0394r/\u0394\u03B8> <\u03942r/\u0394\u03B82> <max/min> <inflex>
   * @param file jmno souboru
   */
  public void save(String file) {
    int size = res.size();
    String str;
    ResultRow lrow, row;

    try {
      BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new
          FileOutputStream(file)));
      str = "# Vystup programu pro analyzu spiral";
      out.write(str, 0, str.length());
      out.newLine();
      str = "# Datum: " + new Date();
      out.write(str, 0, str.length());
      out.newLine();
      if(conf != null) {
        str = "# Parametry: a=" + conf.getA() + ", b=" + conf.getB() + ", ds=" + conf.getDS() + ", fi=" + conf.getFI() + ", A=" + conf.getAM1() + ", dfi=" + conf.getDFI() + ", B=" + conf.getAM2() + ", pomer=" + conf.getPOMER();
        out.write(str, 0, str.length());
        out.newLine();
      }
      str = "# Parametry: dpi=" + eng.gui.getDPI() + ", Hz=" + eng.gui.getHZ();
      out.write(str, 0, str.length());
      out.newLine();
      str = "# crfi = " + crfi + ", c2rfi = " + c2rfi + ", fi = " + fi +
          ", maxr = " + maxr + ", num = " + size;
      out.write(str, 0, str.length());
      out.newLine();
      str = "# I1 = " + I1 + ", I2 = " + I2 + ", I3 = " + I3 + ", I4 = " + I4 +
          ", I5 = " + I5;
      out.write(str, 0, str.length());
      out.newLine();
      str = "# DOS = " + DOS;
      out.write(str, 0, str.length());
      out.newLine();
      str = "# Vystup: x y time pres r fi dfi drfi d2rfi extrem inflex";
      out.write(str, 0, str.length());
      out.newLine();

      lrow = res.get(0);
      str = lrow.toString() + "0\t0";
      out.write(str, 0, str.length());
      out.newLine();
      for (int i = 1; i < size; i++) {
        row = res.get(i);
        str = row.toString();
        str += Math.abs(sign(row.drfi - crfi) - sign(lrow.drfi - crfi)) / 2;
        str += "\t";
        str += Math.abs(sign(row.d2rfi - c2rfi) - sign(lrow.d2rfi - c2rfi)) / 2;
        out.write(str, 0, str.length());
        out.newLine();
        lrow = row;
      }
      out.close();
    }
    catch (IOException ex) {
      eng.gui.mShow("Nelze vytvorit soubor");
    }
    eng.gui.mShow("Data ulozena");
  }

  /**
   * Vrac velikost vypotanho vektoru
   * @return poet hodnot
   */
  public int size() {
    if (res != null) {
      return res.size();
    }
    else {
      return 0;
    }
  }
}

class ResultVector {
  private Vector vec;
  public ResultVector() {
    vec = new Vector(1000);
  }

  public ResultRow lastElement() {
    return (ResultRow) vec.lastElement();
  }

  public ResultVector(int size) {
    vec = new Vector(size);
  }

  public ResultVector(DataVector mv) {
    int size = mv.size();
    vec = new Vector(size);
    for (int i = 0; i < size; i++) {
      vec.add(new DataRow(mv.get(i)));
    }
  }

  public void add(ResultRow r) {
    vec.add(r);
  }

  public int size() {
    return vec.size();
  }

  public ResultRow get(int pos) {
    return (ResultRow) vec.get(pos);
  }
}

class ResultRow {
  double x, y, r, fi, dfi, drfi, d2rfi;
  int time, pres;

  public ResultRow(double x, double y, int time, int pres, double r, double fi,
                   double dfi, double drfi, double d2rfi) {
    this.x = x;
    this.y = y;
    this.time = time;
    this.pres = pres;
    this.r = r;
    this.fi = fi;
    this.dfi = dfi;
    this.drfi = drfi;
    this.d2rfi = d2rfi;
  }

  public String toString() {
    StringBuffer buf = new StringBuffer(110);
    buf.append(round(x));
    buf.append(round(y));
    buf.append(time);
    buf.append("\t");
    buf.append(pres);
    buf.append("\t");
    buf.append(round(r));
    buf.append(round(fi));
    buf.append(round(dfi));
    buf.append(round(drfi));
    buf.append(round(d2rfi));
    return buf.toString();
  }

  private StringBuffer round(double val, int size, int prec) {
    int pow = (int) Math.pow(10, prec - 2);
    StringBuffer res = new StringBuffer(String.valueOf(Math.rint(val * pow) /
        pow));
    for (; (res.length() - res.indexOf(".")) < prec; ) {
      res.append(' ');
    }
    for (; res.length() < size; ) {
      res.insert(0, ' ');
    }
    return res;
  }

  private StringBuffer round(double val, int size) {
    return round(val, size, 7);
  }

  private StringBuffer round(double val) {
    return round(val, 12);
  }
}
