package spiral.digitalize;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.io.File;

/**
 * <p>Title: Spiral analyzis</p>
 * <p>Description: Frame zajiujc komunikaci s uivatelem</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Doleel
 * @version 1.0
 */

public class MainFrame
    extends JFrame
    implements GUI {
  private Engine engine;
  JFileChooser Chooser = new JFileChooser();
  private String path;

  class Panel
      extends JPanel
      implements ImageObserver {
    public void paint(Graphics g) {
      super.paint(g);
      if (engine.getImage() != null) {
        this.setPreferredSize(new Dimension(engine.getImage().getWidth(panel),
                                            engine.getImage().getHeight(panel)));
      }
      g.fill3DRect(0, 0, getWidth(), getHeight(), false);
      if (engine.getImage() != null) {
        g.drawImage(engine.getImage(), 0, 0, null, this);
      }
    }

    public boolean ImageUpdate() {
      this.repaint();
      return true;
    }

    /*    public Dimension getPreferredSize() {
          if(img != null) {
            return new Dimension(img.getWidth(panel), img.getHeight(panel));
          }
          return super.getPreferredSize();
        }*/
  }

  JMenuBar Menu = new JMenuBar();
  JMenu jMenu1 = new JMenu();
  JMenuItem Load = new JMenuItem();
  JMenuItem Save = new JMenuItem();
  JMenuItem Exit = new JMenuItem();
  JMenu jMenu2 = new JMenu();
  JMenuItem FindSpiral = new JMenuItem();
  JMenuItem Skeleton = new JMenuItem();
  JMenuItem Dilate = new JMenuItem();
  JMenuItem Threshold = new JMenuItem();
  JMenuItem AllInOne = new JMenuItem();
  JLabel Status = new JLabel();
  Panel panel = new Panel();
  JScrollPane ScrollPane = new JScrollPane(panel);

  /**
   * Konstruktor tdy
   */
  public MainFrame() {
    engine = new Engine(this);
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    jMenu1.setText("File");
    Load.setText("Load Image");
    Load.addActionListener(new MainFrame_Load_actionAdapter(this));
    Save.setText("Save data");
    Save.addActionListener(new MainFrame_Save_actionAdapter(this));
    Exit.setText("Exit");
    Exit.addActionListener(new MainFrame_Exit_actionAdapter(this));
    jMenu2.setText("Apply");
    Threshold.setText("Threshold");
    Threshold.addActionListener(new MainFrame_Threshold_actionAdapter(this));
    Dilate.setText("Dilate");
    Dilate.addActionListener(new MainFrame_Dilate_actionAdapter(this));
    Skeleton.setText("Skeleton");
    Skeleton.addActionListener(new MainFrame_Skeleton_actionAdapter(this));
    AllInOne.setText("All in one");
    AllInOne.addActionListener(new MainFrame_AllInOne_actionAdapter(this));
    FindSpiral.setText("Find Spiral");
    FindSpiral.addActionListener(new MainFrame_FindSpiral_actionAdapter(this));
    Status.setText("No Image");
    this.addWindowListener(new MainFrame_this_windowAdapter(this));
    this.setJMenuBar(Menu);
    this.setTitle("Spiral digitalization");
    Menu.add(jMenu1);
    Menu.add(jMenu2);
    jMenu1.add(Load);
    jMenu1.add(Save);
    jMenu1.addSeparator();
    jMenu1.add(Exit);
    jMenu2.add(Threshold);
    jMenu2.add(Dilate);
    jMenu2.add(Skeleton);
    jMenu2.add(AllInOne);
    jMenu2.add(FindSpiral);
    this.getContentPane().add(Status, BorderLayout.SOUTH);
    this.getContentPane().add(ScrollPane, BorderLayout.CENTER);
    this.setSize(new Dimension(800, 600));
  }

  void this_windowClosing(WindowEvent e) {
    System.exit(0);
  }

  void Exit_actionPerformed(ActionEvent e) {
    this_windowClosing(null);
  }

  void Load_actionPerformed(ActionEvent e) {
    if (JFileChooser.APPROVE_OPTION == Chooser.showOpenDialog(this)) {
        // Call openFile to attempt to load the image from file into Panel
        path = Chooser.getSelectedFile().getPath();
        engine.loadImage(path);
        if(engine.getImage() != null) {
          panel.setSize(new Dimension(engine.getImage().getWidth(panel),
                                      engine.getImage().getHeight(panel)));
        }
      }
    }

    void Save_actionPerformed(ActionEvent e) {
      File file = null;
      if(path != null) {
        int i = path.lastIndexOf('.');
        if (i > 0 && i < path.length() - 1) {
          path = path.substring(0, i + 1);
          path += "dat";
        }
        else {
          path += ".dat";
        }
        file = new File(path);
      }
      Chooser.setSelectedFile(file);
      if (JFileChooser.APPROVE_OPTION == Chooser.showSaveDialog(this)) {
        engine.saveData(Chooser.getSelectedFile().getPath());
      }
    }

  public void showStatus(String text) {
    Status.setText(text);
  }

  void Threshold_actionPerformed(ActionEvent e) {
    engine.threshold();
  }

  void Dilate_actionPerformed(ActionEvent e) {
    engine.dilate();
  }

  void Skeleton_actionPerformed(ActionEvent e) {
    engine.skeleton();
  }

  void FindSpiral_actionPerformed(ActionEvent e) {
    engine.findSpiral();
  }

  void AllInOne_actionPerformed(ActionEvent e) {
    engine.allInOne();
  }
}

class MainFrame_this_windowAdapter
    extends java.awt.event.WindowAdapter {
  MainFrame adaptee;

  MainFrame_this_windowAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class MainFrame_Exit_actionAdapter
    implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Exit_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.Exit_actionPerformed(e);
  }
}

class MainFrame_Load_actionAdapter
    implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Load_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.Load_actionPerformed(e);
  }
}

class MainFrame_Save_actionAdapter
    implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Save_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.Save_actionPerformed(e);
  }
}

class MainFrame_Threshold_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Threshold_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.Threshold_actionPerformed(e);
  }
}

class MainFrame_Dilate_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Dilate_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.Dilate_actionPerformed(e);
  }
}

class MainFrame_Skeleton_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_Skeleton_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.Skeleton_actionPerformed(e);
  }
}

class MainFrame_FindSpiral_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_FindSpiral_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.FindSpiral_actionPerformed(e);
  }
}

class MainFrame_AllInOne_actionAdapter implements java.awt.event.ActionListener {
  MainFrame adaptee;

  MainFrame_AllInOne_actionAdapter(MainFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.AllInOne_actionPerformed(e);
  }
}
